"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.executeAsync = exports.executeSync = exports.hasWdioSyncSupport = exports.wrapCommand = exports.runFnInFiberContext = exports.runTestInFiberContext = exports.executeHooksWithArgs = void 0;

var _logger = _interopRequireDefault(require("@wdio/logger"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const log = (0, _logger.default)('@wdio/config');

const NOOP = () => {};

let executeHooksWithArgs = async function executeHooksWithArgsShim(hooks, args) {
  if (!Array.isArray(hooks)) {
    hooks = [hooks];
  }

  if (!Array.isArray(args)) {
    args = [args];
  }

  hooks = hooks.map(hook => new Promise(resolve => {
    let result;

    try {
      result = hook.apply(null, args);
    } catch (e) {
      log.error(e.stack);
      return resolve();
    }

    if (result && typeof result.then === 'function') {
      return result.then(resolve, e => {
        log.error(e.stack);
        resolve();
      });
    }

    resolve(result);
  }));
  return Promise.all(hooks);
};

exports.executeHooksWithArgs = executeHooksWithArgs;
let runTestInFiberContext = NOOP;
exports.runTestInFiberContext = runTestInFiberContext;

let runFnInFiberContext = fn => {
  return function (...args) {
    return Promise.resolve(fn.apply(this, args));
  };
};

exports.runFnInFiberContext = runFnInFiberContext;

let wrapCommand = (_, origFn) => origFn;

exports.wrapCommand = wrapCommand;
let hasWdioSyncSupport = false;
exports.hasWdioSyncSupport = hasWdioSyncSupport;

let executeSync = (fn, _, args = []) => fn.apply(void 0, args);

exports.executeSync = executeSync;

let executeAsync = (fn, _, args = []) => fn.apply(void 0, args);

exports.executeAsync = executeAsync;

try {
  const wdioSync = require('@wdio/sync');

  exports.hasWdioSyncSupport = hasWdioSyncSupport = true;
  exports.runFnInFiberContext = runFnInFiberContext = wdioSync.runFnInFiberContext;
  exports.runTestInFiberContext = runTestInFiberContext = wdioSync.runTestInFiberContext;
  exports.wrapCommand = wrapCommand = wdioSync.wrapCommand;
  exports.executeHooksWithArgs = executeHooksWithArgs = wdioSync.executeHooksWithArgs;
  exports.executeSync = executeSync = wdioSync.executeSync;
  exports.executeAsync = executeAsync = wdioSync.executeAsync;
} catch (_unused) {}